#include <ole2.h>
#include "bmpctrl.h"
#define __TRACE
#include <checks.h>

TBmpCtrl::TBmpCtrl(LPUNKNOWN pUnkOuter, void (*pfnDestroyed)())
{
  m_cRef = 0;
  m_pUnkOuter = pUnkOuter;
  m_pDefIUnknown = NULL;
  m_pImpIOleObject = NULL;
  m_pIOleClientSite = NULL;
  m_pIOleAdviseHolder = NULL;
  m_pfnDestroyed = pfnDestroyed;
  m_Clsid = CLSID_TBmpCtrl;
  m_pITINeutral = NULL;
  m_pIUnkStdDisp = NULL;
};

TBmpCtrl::~TBmpCtrl()
{
  
  if (NULL != m_pDefIUnknown){
    m_pDefIUnknown->Release();
    m_pDefIUnknown = NULL;
  }

  if (NULL != m_pIUnkStdDisp){
    m_pIUnkStdDisp->Release();
    m_pIUnkStdDisp = NULL;
  }
    
  if (NULL != m_pITINeutral){
    m_pITINeutral->Release();
    m_pITINeutral = NULL;
  }

  if (NULL != m_pImpIOleObject){
    delete m_pImpIOleObject;
    m_pImpIOleObject = NULL;
  }
};


////////////////////////////////////////////////////////////////////////////
// 
// IUnknown members:  QueryInterface, AddRef, Release
//

STDMETHODIMP TBmpCtrl::QueryInterface(REFIID riid, void** ppv)
{
  TCHAR msg[80];
  wsprintf(msg,TEXT("QueryInterface(riid=%x"), riid.Data1);
  OutputDebugString(msg);
  *ppv = NULL;
  
  if (IID_IUnknown==riid)
    *ppv=this;

  if (IID_IOleObject==riid)
  *ppv=m_pImpIOleObject;

  if (IID_IDispatch==riid || IID_IBmpCtrl==riid)
    return m_pIUnkStdDisp->QueryInterface(IID_IDispatch, ppv);

  if (IID_IViewObject==riid       || IID_IViewObject2==riid ||
      IID_IOleCache==riid         || IID_IOleCache2==riid   ||
      IID_IOleCacheControl==riid  ||
      IID_IDataObject==riid       || IID_IPersistStorage==riid)
    return m_pDefIUnknown->QueryInterface(riid, ppv);

  if (NULL!=*ppv)
  {
    ((LPUNKNOWN)*ppv)->AddRef();
    return NOERROR;
  }

  return E_NOINTERFACE;
}


STDMETHODIMP_(ULONG) TBmpCtrl::AddRef()
{
  return ++m_cRef;
}


STDMETHODIMP_(ULONG) TBmpCtrl::Release()
{
  --m_cRef;

  if (0L!=--m_cRef)
      return m_cRef;

  if (NULL!=m_pfnDestroyed)
      (*m_pfnDestroyed)();

  delete this;

  return 0L;
}


////////////////////////////////////////////////////////////////////////////
// 
// 
//

HRESULT TBmpCtrl::Init()
{
  LPUNKNOWN pIUnknown = this;
  ITypeLib* pITypeLib;
  HRESULT hr;
  
  if (NULL!=m_pUnkOuter)
    pIUnknown = m_pUnkOuter;    

  m_pImpIOleObject = new TImpIOleObject(this, pIUnknown);

  if (NULL==m_pImpIOleObject)
    return E_OUTOFMEMORY;

  //////////////////////////////////////////////////////////////////////
  // Create the Data Cache

  hr=CreateDataCache(this, CLSID_TBmpCtrl,
                     IID_IUnknown, &static_cast<void*>(m_pDefIUnknown));

  if (FAILED(hr)){
    delete m_pImpIOleObject;
    return hr;
  }
  
  hr = GrabFromClipboard();
  
  if (FAILED(hr)){
    delete m_pImpIOleObject;
    m_pDefIUnknown->Release();
    return hr;
  }
  
  //////////////////////////////////////////////////////////////////////
  // Create the Dispatch Interface
  
  hr = LoadRegTypeLib(LIBID_BmpCtrlLibrary, 1, 0, LANG_NEUTRAL, &pITypeLib);
  if (FAILED(hr)){
    delete m_pImpIOleObject;
    m_pDefIUnknown->Release();
    return hr;
  }
  
  hr = pITypeLib->GetTypeInfoOfGuid(IID_IBmpCtrl, &m_pITINeutral);
  pITypeLib->Release();
  if (FAILED(hr)){
    delete m_pImpIOleObject;
    m_pDefIUnknown->Release();
    return hr;
  }
  
  hr = CreateStdDispatch(pIUnknown, static_cast<IBmpCtrl*>(this),
                         m_pITINeutral, &m_pIUnkStdDisp);
  if (FAILED(hr)){
    delete m_pImpIOleObject;
    m_pDefIUnknown->Release();
    m_pITINeutral->Release();
    return hr;
  }
  
  return S_OK;
}


HRESULT STDMETHODCALLTYPE TBmpCtrl::GrabFromClipboard()
{
  unsigned int cf = 0;
  HGLOBAL hMem = NULL;
  HRESULT  hr;
  LPUNKNOWN pIUnknown;
  LPPERSISTSTORAGE pIPersistStorage;
  LPOLECACHE pIOleCache;
  STGMEDIUM stm;
  FORMATETC fe;
  DWORD dwConn; 

  if (!OpenClipboard(NULL))
    return E_FAIL;

  stm.tymed = TYMED_MFPICT;
  stm.pUnkForRelease = NULL;
  hMem = GetClipboardData(CF_METAFILEPICT);
  if (NULL!=hMem)
    cf = CF_METAFILEPICT;
  
  if (0==cf) {
    stm.tymed = TYMED_HGLOBAL;
    hMem = GetClipboardData(CF_DIB);
  
    if (NULL!=hMem)
      cf = CF_DIB;
  }  
  if (0==cf){
    stm.tymed = TYMED_GDI;
    hMem = GetClipboardData(CF_BITMAP);
   
    if (NULL!=hMem)
      cf=CF_BITMAP;
  }
  
  if (0==cf)
  {
    stm.tymed=TYMED_HGLOBAL;
    hMem=GetClipboardData(CF_UNICODETEXT);

    if (NULL!=hMem)
      cf=CF_UNICODETEXT;
  }

  stm.hGlobal = OleDuplicateData(hMem,cf,NULL);
  CloseClipboard();


  //Didn't get anything?  Then we're finished.
  if (0==cf)
      return E_FAIL;

  //This now describes the data we have.
  fe.cfFormat = cf;
  fe.dwAspect = DVASPECT_CONTENT;
  fe.ptd = NULL;
  fe.tymed = stm.tymed;
  fe.lindex = -1;

  //By aggregation rules, after calling QueryInterface on an aggregated 
  //object, we must call Release after using the interface.  See
  //Brockshmidt, chapter 2. 

  m_pDefIUnknown->QueryInterface(IID_IOleCache, 
                                 &static_cast<void*>(pIOleCache));
  pIOleCache->Cache(&fe, ADVF_PRIMEFIRST, &dwConn);

  hr=pIOleCache->SetData(&fe, &stm, TRUE);
  pIOleCache->Release();

  if (FAILED(hr))
  {
    ReleaseStgMedium(&stm);
    return E_FAIL;
  }

  return S_OK;
}

  
BSTR STDMETHODCALLTYPE TBmpCtrl::get_Title(void)
{
  return m_Title;
}

void STDMETHODCALLTYPE TBmpCtrl::put_Title(BSTR newtitle)
{
  SysFreeString(m_Title);
  m_Title = SysAllocString(newtitle);
}
  
