#include <ole2.h>
#include "bmpctrl.h"
#define __TRACE
#include <checks.h>

// Count number of objects and number of locks.

ULONG       g_cObj=0;
ULONG       g_cLock=0;



STDAPI __declspec(dllexport) DllGetClassObject(REFCLSID rclsid, REFIID riid
    , void** ppv)
    {
    TRACE("Inside DllGetClassObject");
    TImpIClassFactory* pCF;
    HRESULT         hr;

    if (CLSID_TBmpCtrl!=rclsid)
        return E_FAIL;

    //Check that we can provide the interface
    if (IID_IUnknown!=riid && IID_IClassFactory!=riid)
        return E_NOINTERFACE;

    //Return our factory's IClassFactory
    pCF=new TImpIClassFactory();

    if (NULL==pCF)
        return ResultFromScode(E_OUTOFMEMORY);

    //If the factory hasn't the interface, delete it
    hr=pCF->QueryInterface(riid, ppv);

    if (FAILED(hr))
        delete pCF;

    return hr;
    }


STDAPI __declspec(dllexport) DllCanUnloadNow()
    {
    SCODE   sc;

    //Our answer is whether there are any object or locks
    sc=(0L==g_cObj && 0L==g_cLock) ? S_OK : S_FALSE;
    return ResultFromScode(sc);
    }


/*
 * ObjectDestroyed
 *
 * Purpose:
 *  Function for the Polyline object to call when it gets destroyed.
 *  Since we're in a DLL we only track the number of objects here
 *  letting DllCanUnloadNow take care of the rest.
 */

void ObjectDestroyed(void)
    {
    g_cObj--;
    return;
    }


/////////////////////////////////////////////////////////////////////////
//
// IClassFactory
//

TImpIClassFactory::TImpIClassFactory(void)
    {
    m_cRef=0L;
    return;
    }

TImpIClassFactory::~TImpIClassFactory(void)
    {
    return;
    }

STDMETHODIMP TImpIClassFactory::QueryInterface(REFIID riid, void** ppv)
    {
    *ppv=NULL;

    if (IID_IUnknown==riid || IID_IClassFactory==riid)
        *ppv=this;

    if (NULL!=*ppv)
        {
        ((LPUNKNOWN)*ppv)->AddRef();
        return NOERROR;
        }

    return E_NOINTERFACE;
    }


STDMETHODIMP_(ULONG) TImpIClassFactory::AddRef(void)
    {
    return ++m_cRef;
    }


STDMETHODIMP_(ULONG) TImpIClassFactory::Release(void)
    {
    if (0!=--m_cRef)
        return m_cRef;

    delete this;
    return 0;
    }


STDMETHODIMP TImpIClassFactory::CreateInstance(LPUNKNOWN pUnkOuter
    , REFIID riid, void** ppvObj)
    {
    TBmpCtrl*        pObj;
    HRESULT            hr;

    *ppvObj=NULL;
    hr=E_OUTOFMEMORY;

    //No aggregation supported
    if (NULL!=pUnkOuter && IID_IUnknown!=riid)
        return CLASS_E_NOAGGREGATION;

    //Create the object passing function to notify on destruction.
    pObj=new TBmpCtrl(pUnkOuter, ObjectDestroyed);

    if (NULL==pObj)
        return hr;

    hr = pObj->Init();
    if (SUCCEEDED(hr))
        hr=pObj->QueryInterface(riid, ppvObj);
    else {
        delete pObj;
        return hr;
    }

    //Kill the object if initial creation or Init failed.
    if (FAILED(hr))
        delete pObj;
    else
        g_cObj++;

    return hr;
    }


STDMETHODIMP TImpIClassFactory::LockServer(BOOL fLock)
    {
    if (fLock)
        g_cLock++;
    else
        g_cLock--;

    return NOERROR;
    }

